@@ -5,6 +5,7 @@
 
 #include "qffmpegmediametadata_p.h"
 #include "qffmpegmediaformatinfo_p.h"
+#include "qffmpegioutils_p.h"
 #include "qiodevice.h"
 #include "qdatetime.h"
 #include "qloggingcategory.h"
@@ -64,10 +65,29 @@ static int streamOrientation(const AVStream *stream)
     return rotation < 0 ? -rotation % 360 : -rotation % 360 + 360;
 }
 
-QVideo::RotationAngle MediaDataHolder::getRotationAngle() const
+
+static bool colorTransferSupportsHdr(const AVStream *stream)
+{
+    if (!stream)
+        return false;
+
+    const AVCodecParameters *codecPar = stream->codecpar;
+    if (!codecPar)
+        return false;
+
+    const QVideoFrameFormat::ColorTransfer colorTransfer = fromAvColorTransfer(codecPar->color_trc);
+
+    // Assume that content is using HDR if the color transfer supports high
+    // dynamic range. The video may still not utilize the extended range,
+    // but we can't determine the actual range without decoding frames.
+    return colorTransfer == QVideoFrameFormat::ColorTransfer_ST2084
+            || colorTransfer == QVideoFrameFormat::ColorTransfer_STD_B67;
+}
+
+QtVideo::Rotation MediaDataHolder::rotation() const
 {
     int orientation = m_metaData.value(QMediaMetaData::Orientation).toInt();
-    return static_cast<QVideo::RotationAngle>(orientation);
+    return static_cast<QtVideo::Rotation>(orientation);
 }
 
 AVFormatContext *MediaDataHolder::avContext()
@@ -96,6 +116,7 @@ static void insertMediaData(QMediaMetaData &metaData, QPlatformMediaPlayer::Trac
         metaData.insert(QMediaMetaData::VideoFrameRate,
                         qreal(stream->avg_frame_rate.num) / qreal(stream->avg_frame_rate.den));
         metaData.insert(QMediaMetaData::Orientation, QVariant::fromValue(streamOrientation(stream)));
+        metaData.insert(QMediaMetaData::HasHdrContent, colorTransferSupportsHdr(stream));
         break;
     case QPlatformMediaPlayer::AudioStream:
         metaData.insert(QMediaMetaData::AudioBitRate, (int)codecPar->bit_rate);
@@ -108,36 +129,6 @@ static void insertMediaData(QMediaMetaData &metaData, QPlatformMediaPlayer::Trac
     }
 };
 
-static int readQIODevice(void *opaque, uint8_t *buf, int buf_size)
-{
-    auto *dev = static_cast<QIODevice *>(opaque);
-    if (dev->atEnd())
-        return AVERROR_EOF;
-    return dev->read(reinterpret_cast<char *>(buf), buf_size);
-}
-
-static int64_t seekQIODevice(void *opaque, int64_t offset, int whence)
-{
-    QIODevice *dev = static_cast<QIODevice *>(opaque);
-
-    if (dev->isSequential())
-        return AVERROR(EINVAL);
-
-    if (whence & AVSEEK_SIZE)
-        return dev->size();
-
-    whence &= ~AVSEEK_FORCE;
-
-    if (whence == SEEK_CUR)
-        offset += dev->pos();
-    else if (whence == SEEK_END)
-        offset += dev->size();
-
-    if (!dev->seek(offset))
-        return AVERROR(EINVAL);
-    return offset;
-}
-
 QPlatformMediaPlayer::TrackType MediaDataHolder::trackTypeFromMediaType(int mediaType)
 {
     switch (mediaType) {
@@ -180,6 +171,10 @@ loadMedia(const QUrl &mediaUrl, QIODevice *stream, const std::shared_ptr<ICancel
     constexpr auto NetworkTimeoutUs = "5000000";
     av_dict_set(dict, "timeout", NetworkTimeoutUs, 0);
 
+    const QByteArray protocolWhitelist = qgetenv("QT_FFMPEG_PROTOCOL_WHITELIST");
+    if (!protocolWhitelist.isNull())
+        av_dict_set(dict, "protocol_whitelist", protocolWhitelist.data(), 0);
+
     context->interrupt_callback.opaque = cancelToken.get();
     context->interrupt_callback.callback = [](void *opaque) {
         const auto *cancelToken = static_cast<const ICancelToken *>(opaque);
@@ -199,7 +194,7 @@ loadMedia(const QUrl &mediaUrl, QIODevice *stream, const std::shared_ptr<ICancel
         auto code = QMediaPlayer::ResourceError;
         if (ret == AVERROR(EACCES))
             code = QMediaPlayer::AccessDeniedError;
-        else if (ret == AVERROR(EINVAL))
+        else if (ret == AVERROR(EINVAL) || ret == AVERROR_INVALIDDATA)
             code = QMediaPlayer::FormatError;
 
         return MediaDataHolder::ContextError{ code, QMediaPlayer::tr("Could not open file") };
@@ -218,6 +213,7 @@ loadMedia(const QUrl &mediaUrl, QIODevice *stream, const std::shared_ptr<ICancel
 #endif
     return context;
 }
+
 } // namespace
 
 MediaDataHolder::Maybe MediaDataHolder::create(const QUrl &url, QIODevice *stream,
