@@ -10,6 +10,7 @@
 #include <QtMultimedia/qcameradevice.h>
 
 #include <array>
+#include <thread>
 
 QT_BEGIN_NAMESPACE
 
@@ -240,21 +241,21 @@ QVideoFrameFormat::PixelFormat QGstStructureView::pixelFormat() const
 
 QGRange<float> QGstStructureView::frameRateRange() const
 {
-    float minRate = 0.;
-    float maxRate = 0.;
-
     if (!structure)
         return { 0.f, 0.f };
 
+    std::optional<float> minRate;
+    std::optional<float> maxRate;
+
     auto extractFraction = [](const GValue *v) -> float {
         return (float)gst_value_get_fraction_numerator(v)
                 / (float)gst_value_get_fraction_denominator(v);
     };
     auto extractFrameRate = [&](const GValue *v) {
         auto insert = [&](float min, float max) {
-            if (max > maxRate)
+            if (!maxRate || max > maxRate)
                 maxRate = max;
-            if (min < minRate)
+            if (!minRate || min < minRate)
                 minRate = min;
         };
 
@@ -262,8 +263,8 @@ QGRange<float> QGstStructureView::frameRateRange() const
             float rate = extractFraction(v);
             insert(rate, rate);
         } else if (GST_VALUE_HOLDS_FRACTION_RANGE(v)) {
-            auto *min = gst_value_get_fraction_range_max(v);
-            auto *max = gst_value_get_fraction_range_max(v);
+            const GValue *min = gst_value_get_fraction_range_min(v);
+            const GValue *max = gst_value_get_fraction_range_max(v);
             insert(extractFraction(min), extractFraction(max));
         }
     };
@@ -287,7 +288,39 @@ QGRange<float> QGstStructureView::frameRateRange() const
         }
     }
 
-    return { minRate, maxRate };
+    if (!minRate || !maxRate)
+        return { 0.f, 0.f };
+
+    return {
+        minRate.value_or(*maxRate),
+        maxRate.value_or(*minRate),
+    };
+}
+
+std::optional<QGRange<QSize>> QGstStructureView::resolutionRange() const
+{
+    if (!structure)
+        return std::nullopt;
+
+    const GValue *width = gst_structure_get_value(structure, "width");
+    const GValue *height = gst_structure_get_value(structure, "height");
+
+    if (!width || !height)
+        return std::nullopt;
+
+    for (const GValue *v : { width, height })
+        if (!GST_VALUE_HOLDS_INT_RANGE(v))
+            return std::nullopt;
+
+    int minWidth = gst_value_get_int_range_min(width);
+    int maxWidth = gst_value_get_int_range_max(width);
+    int minHeight = gst_value_get_int_range_min(height);
+    int maxHeight = gst_value_get_int_range_max(height);
+
+    return QGRange<QSize>{
+        QSize(minWidth, minHeight),
+        QSize(maxWidth, maxHeight),
+    };
 }
 
 QGstreamerMessage QGstStructureView::getMessage()
@@ -356,7 +389,7 @@ std::optional<std::pair<QVideoFrameFormat, GstVideoInfo>> QGstCaps::formatAndVid
                              qt_videoFormatLookup[index].pixelFormat);
 
     if (vidInfo.fps_d > 0)
-        format.setFrameRate(qreal(vidInfo.fps_n) / vidInfo.fps_d);
+        format.setStreamFrameRate(qreal(vidInfo.fps_n) / vidInfo.fps_d);
 
     QVideoFrameFormat::ColorRange range = QVideoFrameFormat::ColorRange_Unknown;
     switch (vidInfo.colorimetry.range) {
@@ -596,6 +629,11 @@ void QGstObject::set(const char *property, const QGstCaps &c)
     g_object_set(get(), property, c.caps(), nullptr);
 }
 
+void QGstObject::set(const char *property, void *object, GDestroyNotify destroyFunction)
+{
+    g_object_set_data_full(qGstCheckedCast<GObject>(get()), property, object, destroyFunction);
+}
+
 QGString QGstObject::getString(const char *property) const
 {
     char *s = nullptr;
@@ -659,13 +697,18 @@ double QGstObject::getDouble(const char *property) const
     return d;
 }
 
-QGstObject QGstObject::getObject(const char *property) const
+QGstObject QGstObject::getGstObject(const char *property) const
 {
     GstObject *o = nullptr;
     g_object_get(get(), property, &o, nullptr);
     return QGstObject(o, HasRef);
 }
 
+void *QGstObject::getObject(const char *property) const
+{
+    return g_object_get_data(qGstCheckedCast<GObject>(get()), property);
+}
+
 QGObjectHandlerConnection QGstObject::connect(const char *name, GCallback callback,
                                               gpointer userData)
 {
@@ -776,6 +819,13 @@ QGstTagListHandle QGstPad::tags() const
     return tagList;
 }
 
+QGString QGstPad::streamId() const
+{
+    return QGString{
+        gst_pad_get_stream_id(pad()),
+    };
+}
+
 std::optional<QPlatformMediaPlayer::TrackType> QGstPad::inferTrackTypeFromName() const
 {
     using namespace Qt::Literals;
@@ -836,6 +886,33 @@ bool QGstPad::sendEvent(GstEvent *event)
     return gst_pad_send_event(pad(), event);
 }
 
+void QGstPad::sendFlushStartStop(bool resetTime)
+{
+    GstEvent *flushStart = gst_event_new_flush_start();
+    gboolean ret = sendEvent(flushStart);
+    if (!ret) {
+        qWarning("failed to send flush-start event");
+        return;
+    }
+
+    GstEvent *flushStop = gst_event_new_flush_stop(resetTime);
+    ret = sendEvent(flushStop);
+    if (!ret)
+        qWarning("failed to send flush-stop event");
+}
+
+void QGstPad::sendFlushIfPaused()
+{
+    using namespace std::chrono_literals;
+
+    GstState state = parent().state(1s);
+
+    if (state != GST_STATE_PAUSED)
+        return;
+
+    sendFlushStartStop(/*resetTime=*/true);
+}
+
 // QGstClock
 
 QGstClock::QGstClock(const QGstObject &o)
@@ -1016,9 +1093,9 @@ bool QGstElement::setStateSync(GstState state, std::chrono::nanoseconds timeout)
 
     if (change != GST_STATE_CHANGE_SUCCESS && change != GST_STATE_CHANGE_NO_PREROLL) {
         qWarning() << "Could not change state of" << name() << "to" << state << change;
-        dumpPipelineGraph("setStatSyncFailure");
+        dumpPipelineGraph("setStateSyncFailure");
     }
-    return change == GST_STATE_CHANGE_SUCCESS;
+    return change == GST_STATE_CHANGE_SUCCESS || change == GST_STATE_CHANGE_NO_PREROLL;
 }
 
 bool QGstElement::syncStateWithParent()
@@ -1040,6 +1117,27 @@ bool QGstElement::finishStateChange(std::chrono::nanoseconds timeout)
     return change == GST_STATE_CHANGE_SUCCESS;
 }
 
+bool QGstElement::hasAsyncStateChange(std::chrono::nanoseconds timeout) const
+{
+    GstState state;
+    GstStateChangeReturn change =
+            gst_element_get_state(element(), &state, nullptr, timeout.count());
+    return change == GST_STATE_CHANGE_ASYNC;
+}
+
+bool QGstElement::waitForAsyncStateChangeComplete(std::chrono::nanoseconds timeout) const
+{
+    using namespace std::chrono_literals;
+    for (;;) {
+        if (!hasAsyncStateChange())
+            return true;
+        timeout -= 10ms;
+        if (timeout < 0ms)
+            return false;
+        std::this_thread::sleep_for(10ms);
+    }
+}
+
 void QGstElement::lockState(bool locked)
 {
     gst_element_set_locked_state(element(), locked);
@@ -1060,6 +1158,64 @@ void QGstElement::sendEos() const
     sendEvent(gst_event_new_eos());
 }
 
+std::optional<std::chrono::nanoseconds> QGstElement::duration() const
+{
+    gint64 d;
+    if (!gst_element_query_duration(element(), GST_FORMAT_TIME, &d)) {
+        qDebug() << "QGstElement: failed to query duration";
+        return std::nullopt;
+    }
+    return std::chrono::nanoseconds{ d };
+}
+
+std::optional<std::chrono::milliseconds> QGstElement::durationInMs() const
+{
+    using namespace std::chrono;
+    auto dur = duration();
+    if (dur)
+        return round<milliseconds>(*dur);
+    return std::nullopt;
+}
+
+std::optional<std::chrono::nanoseconds> QGstElement::position() const
+{
+    QGstQueryHandle &query = positionQuery();
+
+    gint64 pos;
+    if (gst_element_query(element(), query.get())) {
+        gst_query_parse_position(query.get(), nullptr, &pos);
+        return std::chrono::nanoseconds{ pos };
+    }
+
+    qDebug() << "QGstElement: failed to query position";
+    return std::nullopt;
+}
+
+std::optional<std::chrono::milliseconds> QGstElement::positionInMs() const
+{
+    using namespace std::chrono;
+    auto pos = position();
+    if (pos)
+        return round<milliseconds>(*pos);
+    return std::nullopt;
+}
+
+std::optional<bool> QGstElement::canSeek() const
+{
+    QGstQueryHandle query{
+        gst_query_new_seeking(GST_FORMAT_TIME),
+        QGstQueryHandle::HasRef,
+    };
+    gboolean canSeek = false;
+    gst_query_parse_seeking(query.get(), nullptr, &canSeek, nullptr, nullptr);
+
+    if (gst_element_query(element(), query.get())) {
+        gst_query_parse_seeking(query.get(), nullptr, &canSeek, nullptr, nullptr);
+        return canSeek;
+    }
+    return std::nullopt;
+}
+
 GstClockTime QGstElement::baseTime() const
 {
     return gst_element_get_base_time(element());
@@ -1110,6 +1266,17 @@ void QGstElement::dumpPipelineGraph(const char *filename) const
     }
 }
 
+QGstQueryHandle &QGstElement::positionQuery() const
+{
+    if (Q_UNLIKELY(!m_positionQuery))
+        m_positionQuery = QGstQueryHandle{
+            gst_query_new_position(GST_FORMAT_TIME),
+            QGstQueryHandle::HasRef,
+        };
+
+    return m_positionQuery;
+}
+
 // QGstBin
 
 QGstBin QGstBin::create(const char *name)
@@ -1185,7 +1352,7 @@ bool QGstBin::syncChildrenState()
     return gst_bin_sync_children_states(bin());
 }
 
-void QGstBin::dumpGraph(const char *fileNamePrefix)
+void QGstBin::dumpGraph(const char *fileNamePrefix) const
 {
     if (isNull())
         return;
@@ -1201,6 +1368,11 @@ QGstElement QGstBin::findByName(const char *name)
     };
 }
 
+void QGstBin::recalculateLatency()
+{
+    gst_bin_recalculate_latency(bin());
+}
+
 // QGstBaseSink
 
 QGstBaseSink::QGstBaseSink(GstBaseSink *element, RefMode mode)
@@ -1236,8 +1408,6 @@ GstBaseSrc *QGstBaseSrc::baseSrc() const
     return qGstCheckedCast<GstBaseSrc>(element());
 }
 
-#if QT_CONFIG(gstreamer_app)
-
 // QGstAppSink
 
 QGstAppSink::QGstAppSink(GstAppSink *element, RefMode mode)
@@ -1328,8 +1498,6 @@ GstFlowReturn QGstAppSrc::pushBuffer(GstBuffer *buffer)
     return gst_app_src_push_buffer(appSrc(), buffer);
 }
 
-#endif
-
 QString qGstErrorMessageCannotFindElement(std::string_view element)
 {
     return QStringLiteral("Could not find the %1 GStreamer element")
