@@ -25,12 +25,11 @@ class QWindowsFormatInfo;
 
 class QWindowsMediaIntegration : public QPlatformMediaIntegration
 {
+    Q_OBJECT
 public:
     QWindowsMediaIntegration();
     ~QWindowsMediaIntegration();
 
-    QPlatformMediaFormatInfo *formatInfo() override;
-
     QMaybe<QPlatformMediaCaptureSession *> createCaptureSession() override;
 
     QMaybe<QPlatformAudioDecoder *> createAudioDecoder(QAudioDecoder *decoder) override;
@@ -41,7 +40,10 @@ public:
 
     QMaybe<QPlatformVideoSink *> createVideoSink(QVideoSink *sink) override;
 
-    QWindowsFormatInfo *m_formatInfo = nullptr;
+protected:
+    QPlatformMediaFormatInfo *createFormatInfo() override;
+
+    QPlatformVideoDevices *createVideoDevices() override;
 };
 
 QT_END_NAMESPACE
