@@ -21,6 +21,10 @@
 #include <qaudiodevice.h>
 #include <qaudiodecoder.h>
 #include <qaudiobuffer.h>
+#include <qscreencapture.h>
+#include <qwindowcapture.h>
+#include <qaudiobufferinput.h>
+#include <qvideoframeinput.h>
 
 #include <qcamera.h>
 #include <QMediaFormat>
@@ -30,6 +34,8 @@
 
 QT_USE_NAMESPACE
 
+// NOLINTBEGIN(readability-convert-member-functions-to-static)
+
 /*
  This is the backend conformance test.
 
@@ -43,14 +49,6 @@ class tst_QMediaCaptureSession: public QObject
 
 private slots:
 
-    void initTestCase()
-    {
-        if (qEnvironmentVariable("QTEST_ENVIRONMENT").toLower() == "ci") {
-#ifdef Q_OS_ANDROID
-            QSKIP("SKIP initTestCase on CI, because of QTBUG-118571");
-#endif
-        }
-    }
     void testAudioMute();
     void stress_test_setup_and_teardown();
     void stress_test_setup_and_teardown_keep_session();
@@ -67,9 +65,16 @@ private slots:
     void can_change_AudioInput_during_recording();
     void disconnects_deleted_AudioInput();
     void can_move_AudioInput_between_sessions();
+
     void disconnects_deleted_AudioOutput();
     void can_move_AudioOutput_between_sessions_and_player();
 
+    void disconnects_deleted_AudioBufferInput();
+    void can_move_AudioBufferInput_between_sessions();
+
+    void disconnects_deleted_VideoFrameInput();
+    void can_move_VideoFrameInput_between_sessions();
+
     void can_add_and_remove_Camera();
     void can_move_Camera_between_sessions();
     void can_disconnect_Camera_when_recording();
@@ -104,8 +109,8 @@ void tst_QMediaCaptureSession::recordOk(QMediaCaptureSession &session)
     QMediaRecorder recorder;
     session.setRecorder(&recorder);
 
-    QSignalSpy recorderErrorSignal(&recorder, SIGNAL(errorOccurred(Error, const QString &)));
-    QSignalSpy durationChanged(&recorder, SIGNAL(durationChanged(qint64)));
+    QSignalSpy recorderErrorSignal(&recorder, &QMediaRecorder::errorOccurred);
+    QSignalSpy durationChanged(&recorder, &QMediaRecorder::durationChanged);
 
     recorder.record();
     QTRY_VERIFY_WITH_TIMEOUT(recorder.recorderState() == QMediaRecorder::RecordingState, 2000);
@@ -124,7 +129,7 @@ void tst_QMediaCaptureSession::recordOk(QMediaCaptureSession &session)
 void tst_QMediaCaptureSession::recordFail(QMediaCaptureSession &session)
 {
     QMediaRecorder recorder;
-    QSignalSpy recorderErrorSignal(&recorder, SIGNAL(errorOccurred(Error, const QString &)));
+    QSignalSpy recorderErrorSignal(&recorder, &QMediaRecorder::errorOccurred);
 
     session.setRecorder(&recorder);
     recorder.record();
@@ -135,6 +140,8 @@ void tst_QMediaCaptureSession::recordFail(QMediaCaptureSession &session)
 
 void tst_QMediaCaptureSession::stress_test_setup_and_teardown()
 {
+    QSKIP_GSTREAMER("QTBUG-124005: spurious seek failures on CI");
+
     for (int i = 0; i < 50; i++) {
         QMediaCaptureSession session;
         QMediaRecorder recorder;
@@ -143,8 +150,10 @@ void tst_QMediaCaptureSession::stress_test_setup_and_teardown()
         QAudioOutput output;
         QVideoWidget video;
 
-        session.setAudioInput(&input);
-        session.setAudioOutput(&output);
+        if (!input.device().isNull())
+            session.setAudioInput(&input);
+        if (!output.device().isNull())
+            session.setAudioOutput(&output);
         session.setRecorder(&recorder);
         session.setCamera(&camera);
         session.setVideoOutput(&video);
@@ -156,6 +165,8 @@ void tst_QMediaCaptureSession::stress_test_setup_and_teardown()
 
 void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_session()
 {
+    QSKIP_GSTREAMER("QTBUG-124005: spurious seek failures on CI");
+
     QMediaCaptureSession session;
     for (int i = 0; i < 50; i++) {
         QMediaRecorder recorder;
@@ -164,8 +175,10 @@ void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_session()
         QAudioOutput output;
         QVideoWidget video;
 
-        session.setAudioInput(&input);
-        session.setAudioOutput(&output);
+        if (!input.device().isNull())
+            session.setAudioInput(&input);
+        if (!output.device().isNull())
+            session.setAudioOutput(&output);
         session.setRecorder(&recorder);
         session.setCamera(&camera);
         session.setVideoOutput(&video);
@@ -177,6 +190,8 @@ void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_session()
 
 void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_recorder()
 {
+    QSKIP_GSTREAMER("QTBUG-124005: spurious seek failures on CI");
+
     QMediaCaptureSession session;
     QMediaRecorder recorder;
     for (int i = 0; i < 50; i++) {
@@ -185,8 +200,10 @@ void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_recorder()
         QAudioOutput output;
         QVideoWidget video;
 
-        session.setAudioInput(&input);
-        session.setAudioOutput(&output);
+        if (!input.device().isNull())
+            session.setAudioInput(&input);
+        if (!output.device().isNull())
+            session.setAudioOutput(&output);
         session.setRecorder(&recorder);
         session.setCamera(&camera);
         session.setVideoOutput(&video);
@@ -198,6 +215,8 @@ void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_recorder()
 
 void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_camera()
 {
+    QSKIP_GSTREAMER("QTBUG-124005: spurious seek failures on CI");
+
     QCamera camera;
     for (int i = 0; i < 50; i++) {
         QMediaCaptureSession session;
@@ -206,8 +225,10 @@ void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_camera()
         QAudioOutput output;
         QVideoWidget video;
 
-        session.setAudioInput(&input);
-        session.setAudioOutput(&output);
+        if (!input.device().isNull())
+            session.setAudioInput(&input);
+        if (!output.device().isNull())
+            session.setAudioOutput(&output);
         session.setRecorder(&recorder);
         session.setCamera(&camera);
         session.setVideoOutput(&video);
@@ -219,6 +240,8 @@ void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_camera()
 
 void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_audioinput()
 {
+    QSKIP_GSTREAMER("QTBUG-124005: spurious seek failures on CI");
+
     QAudioInput input;
     for (int i = 0; i < 50; i++) {
         QMediaCaptureSession session;
@@ -227,8 +250,10 @@ void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_audioinput()
         QAudioOutput output;
         QVideoWidget video;
 
-        session.setAudioInput(&input);
-        session.setAudioOutput(&output);
+        if (!input.device().isNull())
+            session.setAudioInput(&input);
+        if (!output.device().isNull())
+            session.setAudioOutput(&output);
         session.setRecorder(&recorder);
         session.setCamera(&camera);
         session.setVideoOutput(&video);
@@ -240,6 +265,8 @@ void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_audioinput()
 
 void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_audiooutput()
 {
+    QSKIP_GSTREAMER("QTBUG-124005: spurious seek failures on CI");
+
     QAudioOutput output;
     for (int i = 0; i < 50; i++) {
         QMediaCaptureSession session;
@@ -248,8 +275,10 @@ void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_audiooutput()
         QAudioInput input;
         QVideoWidget video;
 
-        session.setAudioInput(&input);
-        session.setAudioOutput(&output);
+        if (!input.device().isNull())
+            session.setAudioInput(&input);
+        if (!output.device().isNull())
+            session.setAudioOutput(&output);
         session.setRecorder(&recorder);
         session.setCamera(&camera);
         session.setVideoOutput(&video);
@@ -261,6 +290,8 @@ void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_audiooutput()
 
 void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_video()
 {
+    QSKIP_GSTREAMER("QTBUG-124005: spurious seek failures on CI");
+
     QVideoWidget video;
     for (int i = 0; i < 50; i++) {
         QMediaCaptureSession session;
@@ -269,8 +300,10 @@ void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_video()
         QAudioInput input;
         QAudioOutput output;
 
-        session.setAudioInput(&input);
-        session.setAudioOutput(&output);
+        if (!input.device().isNull())
+            session.setAudioInput(&input);
+        if (!output.device().isNull())
+            session.setAudioOutput(&output);
         session.setRecorder(&recorder);
         session.setCamera(&camera);
         session.setVideoOutput(&video);
@@ -292,7 +325,7 @@ void tst_QMediaCaptureSession::record_video_without_preview()
 
     session.setRecorder(&recorder);
 
-    QSignalSpy cameraChanged(&session, SIGNAL(cameraChanged()));
+    QSignalSpy cameraChanged(&session, &QMediaCaptureSession::cameraChanged);
 
     session.setCamera(&camera);
     camera.setActive(true);
@@ -317,8 +350,8 @@ void tst_QMediaCaptureSession::can_add_and_remove_AudioInput_with_and_without_Au
         QSKIP("No audio input available");
 
     QMediaCaptureSession session;
-    QSignalSpy audioInputChanged(&session, SIGNAL(audioInputChanged()));
-    QSignalSpy audioOutputChanged(&session, SIGNAL(audioOutputChanged()));
+    QSignalSpy audioInputChanged(&session, &QMediaCaptureSession::audioInputChanged);
+    QSignalSpy audioOutputChanged(&session, &QMediaCaptureSession::audioOutputChanged);
 
     session.setAudioInput(&input);
     QTRY_COMPARE(audioInputChanged.size(), 1);
@@ -349,10 +382,10 @@ void tst_QMediaCaptureSession::can_change_AudioDevices_on_attached_AudioInput()
         QSKIP("Two audio inputs are not available");
 
     QAudioInput input(audioInputs[0]);
-    QSignalSpy deviceChanged(&input, SIGNAL(deviceChanged()));
+    QSignalSpy deviceChanged(&input, &QAudioInput::deviceChanged);
 
     QMediaCaptureSession session;
-    QSignalSpy audioInputChanged(&session, SIGNAL(audioInputChanged()));
+    QSignalSpy audioInputChanged(&session, &QMediaCaptureSession::audioInputChanged);
 
     session.setAudioInput(&input);
     QTRY_COMPARE(audioInputChanged.size(), 1);
@@ -384,9 +417,9 @@ void tst_QMediaCaptureSession::can_change_AudioInput_during_recording()
 
     session.setRecorder(&recorder);
 
-    QSignalSpy audioInputChanged(&session, SIGNAL(audioInputChanged()));
-    QSignalSpy recorderErrorSignal(&recorder, SIGNAL(errorOccurred(Error, const QString &)));
-    QSignalSpy durationChanged(&recorder, SIGNAL(durationChanged(qint64)));
+    QSignalSpy audioInputChanged(&session, &QMediaCaptureSession::audioInputChanged);
+    QSignalSpy recorderErrorSignal(&recorder, &QMediaRecorder::errorOccurred);
+    QSignalSpy durationChanged(&recorder, &QMediaRecorder::durationChanged);
 
     session.setAudioInput(&input);
     QTRY_COMPARE(audioInputChanged.size(), 1);
@@ -418,7 +451,7 @@ void tst_QMediaCaptureSession::disconnects_deleted_AudioInput()
         QSKIP("No audio input available");
 
     QMediaCaptureSession session;
-    QSignalSpy audioInputChanged(&session, SIGNAL(audioInputChanged()));
+    QSignalSpy audioInputChanged(&session, &QMediaCaptureSession::audioInputChanged);
     {
         QAudioInput input;
         session.setAudioInput(&input);
@@ -435,13 +468,13 @@ void tst_QMediaCaptureSession::can_move_AudioInput_between_sessions()
 
     QMediaCaptureSession session0;
     QMediaCaptureSession session1;
-    QSignalSpy audioInputChanged0(&session0, SIGNAL(audioInputChanged()));
-    QSignalSpy audioInputChanged1(&session1, SIGNAL(audioInputChanged()));
+    QSignalSpy audioInputChanged0(&session0, &QMediaCaptureSession::audioInputChanged);
+    QSignalSpy audioInputChanged1(&session1, &QMediaCaptureSession::audioInputChanged);
 
     QAudioInput input;
     {
         QMediaCaptureSession session2;
-        QSignalSpy audioInputChanged2(&session2, SIGNAL(audioInputChanged()));
+        QSignalSpy audioInputChanged2(&session2, &QMediaCaptureSession::audioInputChanged);
         session2.setAudioInput(&input);
         QTRY_COMPARE(audioInputChanged2.size(), 1);
     }
@@ -462,7 +495,7 @@ void tst_QMediaCaptureSession::disconnects_deleted_AudioOutput()
         QSKIP("No audio output available");
 
     QMediaCaptureSession session;
-    QSignalSpy audioOutputChanged(&session, SIGNAL(audioOutputChanged()));
+    QSignalSpy audioOutputChanged(&session, &QMediaCaptureSession::audioOutputChanged);
     {
         QAudioOutput output;
         session.setAudioOutput(&output);
@@ -482,13 +515,13 @@ void tst_QMediaCaptureSession::can_move_AudioOutput_between_sessions_and_player(
     QMediaCaptureSession session0;
     QMediaCaptureSession session1;
     QMediaPlayer player;
-    QSignalSpy audioOutputChanged0(&session0, SIGNAL(audioOutputChanged()));
-    QSignalSpy audioOutputChanged1(&session1, SIGNAL(audioOutputChanged()));
-    QSignalSpy audioOutputChangedPlayer(&player, SIGNAL(audioOutputChanged()));
+    QSignalSpy audioOutputChanged0(&session0, &QMediaCaptureSession::audioOutputChanged);
+    QSignalSpy audioOutputChanged1(&session1, &QMediaCaptureSession::audioOutputChanged);
+    QSignalSpy audioOutputChangedPlayer(&player, &QMediaPlayer::audioOutputChanged);
 
     {
         QMediaCaptureSession session2;
-        QSignalSpy audioOutputChanged2(&session2, SIGNAL(audioOutputChanged()));
+        QSignalSpy audioOutputChanged2(&session2, &QMediaCaptureSession::audioOutputChanged);
         session2.setAudioOutput(&output);
         QTRY_COMPARE(audioOutputChanged2.size(), 1);
     }
@@ -518,6 +551,89 @@ void tst_QMediaCaptureSession::can_move_AudioOutput_between_sessions_and_player(
     QVERIFY(player.audioOutput() == nullptr);
 }
 
+void tst_QMediaCaptureSession::disconnects_deleted_AudioBufferInput()
+{
+    QMediaCaptureSession session;
+    QSignalSpy audioBufferInputChanged(&session, &QMediaCaptureSession::audioBufferInputChanged);
+    {
+        QAudioBufferInput input;
+        session.setAudioBufferInput(&input);
+        QTRY_COMPARE(audioBufferInputChanged.size(), 1);
+    }
+    QCOMPARE(session.audioBufferInput(), nullptr);
+    QCOMPARE(audioBufferInputChanged.size(), 2);
+}
+
+void tst_QMediaCaptureSession::can_move_AudioBufferInput_between_sessions()
+{
+    QMediaCaptureSession session0;
+    QMediaCaptureSession session1;
+    QSignalSpy audioBufferInputChanged0(&session0, &QMediaCaptureSession::audioBufferInputChanged);
+    QSignalSpy audioBufferInputChanged1(&session1, &QMediaCaptureSession::audioBufferInputChanged);
+
+    QAudioBufferInput input;
+    {
+        QMediaCaptureSession session2;
+        QSignalSpy audioBufferInputChanged2(&session2,
+                                            &QMediaCaptureSession::audioBufferInputChanged);
+        session2.setAudioBufferInput(&input);
+        QCOMPARE(audioBufferInputChanged2.size(), 1);
+    }
+    session0.setAudioBufferInput(&input);
+    QCOMPARE(audioBufferInputChanged0.size(), 1);
+    QCOMPARE(session0.audioBufferInput(), &input);
+    QCOMPARE(input.captureSession(), &session0);
+
+    session1.setAudioBufferInput(&input);
+
+    QCOMPARE(audioBufferInputChanged0.size(), 2);
+    QCOMPARE(session0.audioBufferInput(), nullptr);
+    QCOMPARE(audioBufferInputChanged1.size(), 1);
+    QCOMPARE(session1.audioBufferInput(), &input);
+    QCOMPARE(input.captureSession(), &session1);
+}
+
+void tst_QMediaCaptureSession::disconnects_deleted_VideoFrameInput()
+{
+    QMediaCaptureSession session;
+    QSignalSpy videoFrameInputChanged(&session, &QMediaCaptureSession::videoFrameInputChanged);
+    {
+        QVideoFrameInput input;
+        session.setVideoFrameInput(&input);
+        QTRY_COMPARE(videoFrameInputChanged.size(), 1);
+    }
+    QCOMPARE(session.videoFrameInput(), nullptr);
+    QCOMPARE(videoFrameInputChanged.size(), 2);
+}
+
+void tst_QMediaCaptureSession::can_move_VideoFrameInput_between_sessions()
+{
+    QMediaCaptureSession session0;
+    QMediaCaptureSession session1;
+    QSignalSpy videoFrameInputChanged0(&session0, &QMediaCaptureSession::videoFrameInputChanged);
+    QSignalSpy videoFrameInputChanged1(&session1, &QMediaCaptureSession::videoFrameInputChanged);
+
+    QVideoFrameInput input;
+    {
+        QMediaCaptureSession session2;
+        QSignalSpy videoFrameInputChanged2(&session2,
+                                           &QMediaCaptureSession::videoFrameInputChanged);
+        session2.setVideoFrameInput(&input);
+        QCOMPARE(videoFrameInputChanged2.size(), 1);
+    }
+    session0.setVideoFrameInput(&input);
+    QCOMPARE(videoFrameInputChanged0.size(), 1);
+    QCOMPARE(session0.videoFrameInput(), &input);
+    QCOMPARE(input.captureSession(), &session0);
+
+    session1.setVideoFrameInput(&input);
+
+    QCOMPARE(videoFrameInputChanged0.size(), 2);
+    QCOMPARE(session0.videoFrameInput(), nullptr);
+    QCOMPARE(videoFrameInputChanged1.size(), 1);
+    QCOMPARE(session1.videoFrameInput(), &input);
+    QCOMPARE(input.captureSession(), &session1);
+}
 
 void tst_QMediaCaptureSession::can_add_and_remove_Camera()
 {
@@ -531,7 +647,7 @@ void tst_QMediaCaptureSession::can_add_and_remove_Camera()
 
     session.setRecorder(&recorder);
 
-    QSignalSpy cameraChanged(&session, SIGNAL(cameraChanged()));
+    QSignalSpy cameraChanged(&session, &QMediaCaptureSession::cameraChanged);
 
     session.setCamera(&camera);
     camera.setActive(true);
@@ -552,13 +668,13 @@ void tst_QMediaCaptureSession::can_move_Camera_between_sessions()
 {
     QMediaCaptureSession session0;
     QMediaCaptureSession session1;
-    QSignalSpy cameraChanged0(&session0, SIGNAL(cameraChanged()));
-    QSignalSpy cameraChanged1(&session1, SIGNAL(cameraChanged()));
+    QSignalSpy cameraChanged0(&session0, &QMediaCaptureSession::cameraChanged);
+    QSignalSpy cameraChanged1(&session1, &QMediaCaptureSession::cameraChanged);
     {
         QCamera camera;
         {
             QMediaCaptureSession session2;
-            QSignalSpy cameraChanged2(&session2, SIGNAL(cameraChanged()));
+            QSignalSpy cameraChanged2(&session2, &QMediaCaptureSession::cameraChanged);
             session2.setCamera(&camera);
             QTRY_COMPARE(cameraChanged2.size(), 1);
         }
@@ -592,9 +708,9 @@ void tst_QMediaCaptureSession::can_disconnect_Camera_when_recording()
 
     session.setRecorder(&recorder);
 
-    QSignalSpy cameraChanged(&session, SIGNAL(cameraChanged()));
-    QSignalSpy recorderErrorSignal(&recorder, SIGNAL(errorOccurred(Error, const QString &)));
-    QSignalSpy durationChanged(&recorder, SIGNAL(durationChanged(qint64)));
+    QSignalSpy cameraChanged(&session, &QMediaCaptureSession::cameraChanged);
+    QSignalSpy recorderErrorSignal(&recorder, &QMediaRecorder::errorOccurred);
+    QSignalSpy durationChanged(&recorder, &QMediaRecorder::durationChanged);
 
     camera.setActive(true);
     session.setCamera(&camera);
@@ -634,7 +750,7 @@ void tst_QMediaCaptureSession::can_add_and_remove_different_Cameras()
 
     session.setRecorder(&recorder);
 
-    QSignalSpy cameraChanged(&session, SIGNAL(cameraChanged()));
+    QSignalSpy cameraChanged(&session, &QMediaCaptureSession::cameraChanged);
 
     camera.setActive(true);
     session.setCamera(&camera);
@@ -667,8 +783,8 @@ void tst_QMediaCaptureSession::can_change_CameraDevice_on_attached_Camera()
 
     session.setRecorder(&recorder);
 
-    QSignalSpy cameraDeviceChanged(&camera, SIGNAL(cameraDeviceChanged()));
-    QSignalSpy cameraChanged(&session, SIGNAL(cameraChanged()));
+    QSignalSpy cameraDeviceChanged(&camera, &QCamera::cameraDeviceChanged);
+    QSignalSpy cameraChanged(&session, &QMediaCaptureSession::cameraChanged);
 
     session.setCamera(&camera);
     QTRY_COMPARE(cameraChanged.size(), 1);
@@ -704,8 +820,8 @@ void tst_QMediaCaptureSession::can_change_VideoOutput_with_and_without_camera()
 
     QMediaCaptureSession session;
 
-    QSignalSpy videoOutputChanged(&session, SIGNAL(videoOutputChanged()));
-    QSignalSpy cameraChanged(&session, SIGNAL(cameraChanged()));
+    QSignalSpy videoOutputChanged(&session, &QMediaCaptureSession::videoOutputChanged);
+    QSignalSpy cameraChanged(&session, &QMediaCaptureSession::cameraChanged);
 
     session.setCamera(&camera);
     QTRY_COMPARE(cameraChanged.size(), 1);
@@ -740,10 +856,10 @@ void tst_QMediaCaptureSession::can_change_VideoOutput_when_recording()
 
     session.setRecorder(&recorder);
 
-    QSignalSpy cameraChanged(&session, SIGNAL(cameraChanged()));
-    QSignalSpy recorderErrorSignal(&recorder, SIGNAL(errorOccurred(Error, const QString &)));
-    QSignalSpy durationChanged(&recorder, SIGNAL(durationChanged(qint64)));
-    QSignalSpy videoOutputChanged(&session, SIGNAL(videoOutputChanged()));
+    QSignalSpy cameraChanged(&session, &QMediaCaptureSession::cameraChanged);
+    QSignalSpy recorderErrorSignal(&recorder, &QMediaRecorder::errorOccurred);
+    QSignalSpy durationChanged(&recorder, &QMediaRecorder::durationChanged);
+    QSignalSpy videoOutputChanged(&session, &QMediaCaptureSession::videoOutputChanged);
 
     camera.setActive(true);
     session.setCamera(&camera);
@@ -783,8 +899,8 @@ void tst_QMediaCaptureSession::can_add_and_remove_recorders()
     QMediaRecorder recorder2;
     QMediaCaptureSession session;
 
-    QSignalSpy audioInputChanged(&session, SIGNAL(audioInputChanged()));
-    QSignalSpy recorderChanged(&session, SIGNAL(recorderChanged()));
+    QSignalSpy audioInputChanged(&session, &QMediaCaptureSession::audioInputChanged);
+    QSignalSpy recorderChanged(&session, &QMediaCaptureSession::recorderChanged);
 
     session.setAudioInput(&input);
     QTRY_COMPARE(audioInputChanged.size(), 1);
@@ -806,13 +922,13 @@ void tst_QMediaCaptureSession::can_move_Recorder_between_sessions()
 {
     QMediaCaptureSession session0;
     QMediaCaptureSession session1;
-    QSignalSpy recorderChanged0(&session0, SIGNAL(recorderChanged()));
-    QSignalSpy recorderChanged1(&session1, SIGNAL(recorderChanged()));
+    QSignalSpy recorderChanged0(&session0, &QMediaCaptureSession::recorderChanged);
+    QSignalSpy recorderChanged1(&session1, &QMediaCaptureSession::recorderChanged);
     {
         QMediaRecorder recorder;
         {
             QMediaCaptureSession session2;
-            QSignalSpy recorderChanged2(&session2, SIGNAL(recorderChanged()));
+            QSignalSpy recorderChanged2(&session2, &QMediaCaptureSession::recorderChanged);
             session2.setRecorder(&recorder);
             QTRY_COMPARE(recorderChanged2.size(), 1);
         }
@@ -849,7 +965,7 @@ void tst_QMediaCaptureSession::can_record_AudioInput_with_null_AudioDevice()
     QAudioInput input(nullDevice);
 
     QMediaCaptureSession session;
-    QSignalSpy audioInputChanged(&session, SIGNAL(audioInputChanged()));
+    QSignalSpy audioInputChanged(&session, &QMediaCaptureSession::audioInputChanged);
 
     session.setAudioInput(&input);
     QTRY_COMPARE(audioInputChanged.size(), 1);
@@ -867,7 +983,7 @@ void tst_QMediaCaptureSession::can_record_Camera_with_null_CameraDevice()
     QCamera camera(nullDevice);
 
     QMediaCaptureSession session;
-    QSignalSpy cameraChanged(&session, SIGNAL(cameraChanged()));
+    QSignalSpy cameraChanged(&session, &QMediaCaptureSession::cameraChanged);
 
     session.setCamera(&camera);
     QTRY_COMPARE(cameraChanged.size(), 1);
@@ -888,10 +1004,10 @@ void tst_QMediaCaptureSession::recording_stops_when_recorder_removed()
     QMediaRecorder recorder;
     QMediaCaptureSession session;
 
-    QSignalSpy audioInputChanged(&session, SIGNAL(audioInputChanged()));
-    QSignalSpy recorderChanged(&session, SIGNAL(recorderChanged()));
-    QSignalSpy recorderErrorSignal(&recorder, SIGNAL(errorOccurred(Error, const QString &)));
-    QSignalSpy durationChanged(&recorder, SIGNAL(durationChanged(qint64)));
+    QSignalSpy audioInputChanged(&session, &QMediaCaptureSession::audioInputChanged);
+    QSignalSpy recorderChanged(&session, &QMediaCaptureSession::recorderChanged);
+    QSignalSpy recorderErrorSignal(&recorder, &QMediaRecorder::errorOccurred);
+    QSignalSpy durationChanged(&recorder, &QMediaRecorder::durationChanged);
 
     session.setAudioInput(&input);
     QTRY_COMPARE(audioInputChanged.size(), 1);
@@ -925,9 +1041,9 @@ void tst_QMediaCaptureSession::can_add_and_remove_ImageCapture()
     QImageCapture capture;
     QMediaCaptureSession session;
 
-    QSignalSpy cameraChanged(&session, SIGNAL(cameraChanged()));
-    QSignalSpy imageCaptureChanged(&session, SIGNAL(imageCaptureChanged()));
-    QSignalSpy readyForCaptureChanged(&capture, SIGNAL(readyForCaptureChanged(bool)));
+    QSignalSpy cameraChanged(&session, &QMediaCaptureSession::cameraChanged);
+    QSignalSpy imageCaptureChanged(&session, &QMediaCaptureSession::imageCaptureChanged);
+    QSignalSpy readyForCaptureChanged(&capture, &QImageCapture::readyForCaptureChanged);
 
     QVERIFY(!capture.isAvailable());
     QVERIFY(!capture.isReadyForCapture());
@@ -965,15 +1081,17 @@ void tst_QMediaCaptureSession::can_add_and_remove_ImageCapture()
 
 void tst_QMediaCaptureSession::can_move_ImageCapture_between_sessions()
 {
+    QSKIP_GSTREAMER("QTBUG-124005: Spurious failure on CI");
+
     QMediaCaptureSession session0;
     QMediaCaptureSession session1;
-    QSignalSpy imageCaptureChanged0(&session0, SIGNAL(imageCaptureChanged()));
-    QSignalSpy imageCaptureChanged1(&session1, SIGNAL(imageCaptureChanged()));
+    QSignalSpy imageCaptureChanged0(&session0, &QMediaCaptureSession::imageCaptureChanged);
+    QSignalSpy imageCaptureChanged1(&session1, &QMediaCaptureSession::imageCaptureChanged);
     {
         QImageCapture imageCapture;
         {
             QMediaCaptureSession session2;
-            QSignalSpy imageCaptureChanged2(&session2, SIGNAL(imageCaptureChanged()));
+            QSignalSpy imageCaptureChanged2(&session2, &QMediaCaptureSession::imageCaptureChanged);
             session2.setImageCapture(&imageCapture);
             QTRY_COMPARE(imageCaptureChanged2.size(), 1);
         }
@@ -995,7 +1113,6 @@ void tst_QMediaCaptureSession::can_move_ImageCapture_between_sessions()
     QVERIFY(session1.imageCapture() == nullptr);
 }
 
-
 void tst_QMediaCaptureSession::capture_is_not_available_when_Camera_is_null()
 {
     QCamera camera;
@@ -1006,9 +1123,9 @@ void tst_QMediaCaptureSession::capture_is_not_available_when_Camera_is_null()
     QImageCapture capture;
     QMediaCaptureSession session;
 
-    QSignalSpy cameraChanged(&session, SIGNAL(cameraChanged()));
-    QSignalSpy capturedSignal(&capture, SIGNAL(imageCaptured(int,QImage)));
-    QSignalSpy readyForCaptureChanged(&capture, SIGNAL(readyForCaptureChanged(bool)));
+    QSignalSpy cameraChanged(&session, &QMediaCaptureSession::cameraChanged);
+    QSignalSpy capturedSignal(&capture, &QImageCapture::imageCaptured);
+    QSignalSpy readyForCaptureChanged(&capture, &QImageCapture::readyForCaptureChanged);
 
     session.setImageCapture(&capture);
     session.setCamera(&camera);
@@ -1043,12 +1160,12 @@ void tst_QMediaCaptureSession::can_add_ImageCapture_and_capture_during_recording
     QMediaCaptureSession session;
     QMediaRecorder recorder;
 
-    QSignalSpy recorderChanged(&session, SIGNAL(recorderChanged()));
-    QSignalSpy recorderErrorSignal(&recorder, SIGNAL(errorOccurred(Error, const QString &)));
-    QSignalSpy durationChanged(&recorder, SIGNAL(durationChanged(qint64)));
-    QSignalSpy imageCaptureChanged(&session, SIGNAL(imageCaptureChanged()));
-    QSignalSpy readyForCaptureChanged(&capture, SIGNAL(readyForCaptureChanged(bool)));
-    QSignalSpy capturedSignal(&capture, SIGNAL(imageCaptured(int,QImage)));
+    QSignalSpy recorderChanged(&session, &QMediaCaptureSession::recorderChanged);
+    QSignalSpy recorderErrorSignal(&recorder, &QMediaRecorder::errorOccurred);
+    QSignalSpy durationChanged(&recorder, &QMediaRecorder::durationChanged);
+    QSignalSpy imageCaptureChanged(&session, &QMediaCaptureSession::imageCaptureChanged);
+    QSignalSpy readyForCaptureChanged(&capture, &QImageCapture::readyForCaptureChanged);
+    QSignalSpy capturedSignal(&capture, &QImageCapture::imageCaptured);
 
     session.setCamera(&camera);
     camera.setActive(true);
@@ -1100,7 +1217,7 @@ void tst_QMediaCaptureSession::testAudioMute()
     recorder.setOutputLocation(QStringLiteral("test"));
 
     QSignalSpy spy(&audioInput, &QAudioInput::mutedChanged);
-    QSignalSpy durationChanged(&recorder, SIGNAL(durationChanged(qint64)));
+    QSignalSpy durationChanged(&recorder, &QMediaRecorder::durationChanged);
 
     QMediaFormat format;
     format.setAudioCodec(QMediaFormat::AudioCodec::Wave);
